{
This TTimer replacement uses the high resolution timing capabilities of
the MMSystem to provide timing far more precise than those possible with
standard Windows's timer resources.  It also illustrates the use of
Delphi 2.0's  TThread class, and the use of a Win32 event for thread
synchronization.   Freeware with source code.

Provided by
Silicon Commander Games
http://www.silicmdr.com

JAB : added a frames-per-second property for convenience & modified interval set.
Modified ontimer call to addSynchronize(...) : removed sender param. Need
Synchronize for Zonerings, becuase GDI handles are freed etc.
Modified Thread creation/destruction to plug memory leak in (???) VCL (???)
                               
}

unit Hirestim;

interface

uses
	Windows, MMSystem, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type

  THiResTimer = class;
  EHiResTimer = class( Exception );

	TTimerThread = class( TThread )
  private
  protected
  public
     hr: THiResTimer;
		 procedure Execute; override;
  end;

  THiResTimer = class( TComponent )
	private
		 nID					: UINT;
		 FEnabled			: boolean;
		 FInterval		: UINT;
		 FFPS					: integer;
		 FResolution	: UINT;
		 FOnTimer			: TThreadMethod;
		 hTimerEvent	: THandle;
		 bPaused			: boolean;
		 timerThread	: TTimerThread;
		 procedure CreateTimer;
	protected
		 procedure SetEnabled(b:boolean);
		 procedure SetFPS(fps_:integer);
		 procedure SetInterval(i:UINT);
	public
		 constructor	Create( AOwner: TComponent ); override;
		 destructor		Destroy; override;
		 procedure		Pause;
		 procedure		Resume;
	published
		 property Enabled: boolean read FEnabled write SetEnabled default FALSE;
		 property Interval: UINT read FInterval write SetInterval default 100;
		 property FPS:integer read FFPS write SetFPS default 10;
     property Resolution: UINT read FResolution write FResolution default 100;
     property OnTimer: TThreadMethod read FOnTimer write FOnTimer;
  end;

procedure Register;

implementation

procedure TimerCallback( uTimerID, uMessage: UINT; dwUser, dw1, dw2: DWORD ); stdcall;
var hr: THiResTimer;
begin
	hr := THiResTimer( dwUser );
	if hr <> nil then if not hr.bPaused then SetEvent(hr.hTimerEvent);
end;

procedure TTimerThread.Execute;
begin
	while not Terminated and ( hr <> nil ) do begin
		WaitForSingleObject( hr.hTimerEvent, INFINITE );
		if Assigned( hr.FOnTimer ) then Synchronize(hr.FOnTimer);
	end;
end;

constructor THiResTimer.Create( AOwner: TComponent );
begin
	inherited Create( AOwner );
	FEnabled										:= FALSE;
	FInterval										:= 100;
	FFPS												:= 10;
	FResolution									:= 100;
	bPaused											:= false;
	timerThread									:= TTimerThread.Create( TRUE );
	timerThread.hr							:= self;
	timerThread.FreeOnTerminate := False;
	hTimerEvent 								:= CreateEvent( nil, FALSE, FALSE, nil );
end;

destructor THiResTimer.Destroy;
begin
	Enabled := FALSE;
	timerThread.Terminate;
	timerThread.Destroy;
	CloseHandle( hTimerEvent );
	inherited Destroy;
end;

procedure THiResTimer.SetEnabled( b: boolean );
begin

	if (csDesigning in ComponentState) then begin
		if b then ShowMessage( 'Set to True in Form''s OnShow event' );
		FEnabled := false;
		Exit;
	end;

	if b <> FEnabled then begin
		if b then begin
			CreateTimer;
			timerThread.Resume;
		end
		else begin
			timerThread.suspend;
			if (timeKillEvent( nID )<>TIMERR_NOERROR) then raise
				EHiResTimer.Create( 'Timer event not freed !' );
		end;
		FEnabled := b;
	end;
end;

procedure THiResTimer.CreateTimer;
var lpTimerProc : TFNTimeCallBack;
begin
	lpTimerProc := @TimerCallback;
	nID := timeSetEvent( FInterval, FResolution, lpTimerProc, DWORD( self ), TIME_PERIODIC );
	if nID = 0 then begin
		FEnabled := FALSE;
		raise EHiResTimer.Create( 'Unable to create a timer' );
	end;
end;

procedure THiResTimer.Pause;
begin
	bPaused := true;
	if Enabled then timerThread.Suspend;
end;

procedure THiResTimer.Resume;
begin
	bPaused := false;
	if Enabled then timerThread.Resume;
end;
/////////////////////////////////////////
// Added by JAB
/////////////////////////////////////////
procedure THiResTimer.SetFPS(FPS_:integer);
begin
	FFPS := FPS_;
	Interval := 1000 div FPS_;
end;

procedure THiResTimer.SetInterval(i:UINT);
begin
	if Enabled then begin
		Enabled		:= false;
		FInterval	:= i;
		Enabled		:= true;
	end else FInterval := i;
end;

/////////////////////////////////////////
// end
/////////////////////////////////////////
procedure Register;
begin
	RegisterComponents( 'Custom', [THiResTimer] );
end;

end.
