///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                    Object to handle compressed I/O stream                 //
//                             John Biddiscombe                             //
//                           j.biddiscombe@rl.ac.uk                          //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
{
Thanks to the following for the LZH unit

 * LZSS coded by Haruhiko OKUMURA
 * Adaptive Huffman Coding coded by Haruyasu YOSHIZAKI
 * Edited and translated to English by Kenji RIKITAKE
 * Translated from C to Turbo Pascal by Douglas Webb   2/18/91
 *    Update and bug correction of TP version 4/29/91 (Sorry!!)
 *    Added Delphi exception handling may 09 1996 Danny Heijl
 *                                                Danny.Heijl@cevi.be

}

unit My_LZH;

interface

uses
  { Borland }
  Classes, ComCtrls,
  { Downloaded }
  LZH;

type
  My_LZH_Compressor = Class
    i_s,in_Stream  : TMemoryStream;
    o_s,out_Stream : TMemoryStream;
    getproc        : GetBytesProc;
    putproc        : PutBytesProc;
    Bytes_done     : integer;
    progress       : integer;
    n_progress     : TNotifyEvent;
    progressbar    : TProgressBar;
    constructor Create;
    Destructor  Destroy; override;
    procedure   Get_Data(VAR DTA; NBytes:WORD; VAR Bytes_Got:WORD);
    procedure   Put_Data(VAR DTA; NBytes:WORD; VAR Bytes_Put:WORD);
    procedure   compression(dir:boolean);
  end;

implementation

constructor My_LZH_Compressor.Create;
begin
  i_s     := TMemoryStream.Create;
  o_s     := TMemoryStream.Create;
  getproc := Get_Data;
  putproc := Put_Data;
end;

Destructor My_LZH_Compressor.Destroy;
begin
  i_s.Free;
  o_s.Free;
end;

procedure My_LZH_Compressor.Get_Data(VAR DTA; NBytes:WORD; VAR Bytes_Got:WORD);
var temp : integer;
begin
  Bytes_Got  := in_stream.Read(DTA,NBytes);
  Bytes_done := Bytes_done + Bytes_Got;

  temp       := progress;
  progress   := bytes_done shr 10;
  if (progress<>temp) then begin
    if assigned(n_progress)  then n_progress(pointer(progress));
    if assigned(progressbar) then progressbar.Position := progress;
  end;
end;

procedure My_LZH_Compressor.Put_Data(VAR DTA; NBytes:WORD; VAR Bytes_Put:WORD);
begin
  Bytes_Put  := out_stream.Write(DTA,NBytes);
end;

procedure My_LZH_Compressor.compression(dir:boolean);
var total_bytes : longint;
begin
  if dir then begin
    in_stream  := i_s;
    out_stream := o_s;
  end else begin
    in_stream  := o_s;
    out_stream := i_s;
  end;
  Bytes_done          := 0;
  progress            := 0;
  in_stream.Position  := 0;
  out_stream.Clear;
  if dir then begin
    total_bytes := in_stream.size;
    out_stream.Write(total_bytes,sizeof(integer));
    LZHPack(total_bytes,getproc,putproc);
  end else begin
    in_stream.Read(total_bytes,sizeof(integer));
    LZHUnPack(total_bytes,getproc,putproc);
  end;
  in_stream.Clear;
end;

end.
